#include <windows.h>
#include "avisynth.h"

class PseudoColor : public GenericVideoFilter {


public: 
	int m_RDiv,m_GDiv,m_BDiv;
PseudoColor(PClip _child, int RDiv,int GDiv,int BDiv) : GenericVideoFilter(_child) 
	{m_RDiv=RDiv;m_GDiv=GDiv;m_BDiv=BDiv; }; 
PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env); 
};

PVideoFrame __stdcall PseudoColor::GetFrame(int n, IScriptEnvironment* env) {

PVideoFrame src = child->GetFrame(n, env); 
PVideoFrame dst = env->NewVideoFrame(vi); 


const unsigned char* srcpY = src->GetReadPtr(PLANAR_Y); 
const unsigned char* srcpV = src->GetReadPtr(PLANAR_V); 
const unsigned char* srcpU = src->GetReadPtr(PLANAR_U); 


unsigned char* dstpY = dst->GetWritePtr(PLANAR_Y); 
unsigned char* dstpV = dst->GetWritePtr(PLANAR_V); 
unsigned char* dstpU = dst->GetWritePtr(PLANAR_U); 


const int src_pitchY = src->GetPitch(PLANAR_Y); 
const int src_pitchUV = src->GetPitch(PLANAR_V); 
const int dst_pitchY = dst->GetPitch(PLANAR_Y); 
const int dst_pitchUV = dst->GetPitch(PLANAR_U); 


const int row_sizeY = dst->GetRowSize(PLANAR_Y); // Could also be PLANAR_Y_ALIGNED which would return a mod16 rowsize 
const int row_sizeUV = dst->GetRowSize(PLANAR_U); // Could also be PLANAR_U_ALIGNED which would return a mod8 rowsize 


const int heightY = dst->GetHeight(PLANAR_Y); 
const int heightUV = dst->GetHeight(PLANAR_U); 

int processed=0;

if (GetVideoInfo().IsRGB32())
{
	processed=1;
	for (int y = 0; y < heightY; y++) 
	{
		for (int x = 0; x < row_sizeY; x+=4)
		{
			int is = (( (  66 * srcpY[x+2] + 129 * srcpY[x+1] +  25 * srcpY[x] + 128) >> 8) +  16)*128;
			dstpY[x] = (is/m_BDiv); 
			dstpY[x+1]=(is/m_GDiv);
			dstpY[x+2]=(is/m_RDiv);
			dstpY[x+3]=srcpY[x+3] ;

		}
		srcpY += src_pitchY; 
		dstpY += dst_pitchY; 
	} 
}
if (GetVideoInfo().IsRGB24())
{
	processed=1;
	for (int y = 0; y < heightY; y++) 
	{
		for (int x = 0; x < row_sizeY; x+=3)
		{
			int is = (((  66 * srcpY[x+2] + 129 * srcpY[x+1] +  25 * srcpY[x] + 128) >> 8) +  16)*128;
			dstpY[x] = (is/m_BDiv); 
			dstpY[x+1]=(is/m_GDiv);
			dstpY[x+2]=(is/m_RDiv);
		}
		srcpY += src_pitchY; 
		dstpY += dst_pitchY; 
	} 
}

if (GetVideoInfo().IsYV12())
{
//Y = ( (  66 * R + 129 * G +  25 * B + 128) >> 8) +  16
//U = ( ( -38 * R -  74 * G + 112 * B + 128) >> 8) + 128
//V = ( ( 112 * R -  94 * G -  18 * B + 128) >> 8) + 128

	processed=1;
	for (int y = 0; y < heightUV; y++) 
	{
		for (int x = 0; x < row_sizeUV; x++) 
		{
			int is = srcpY[(x*row_sizeY)/row_sizeUV+(y*heightY*src_pitchY)/heightUV]*128;
			dstpU[x] = ( ( -38 * is/m_RDiv -  74 * is/m_GDiv + 112 * is/m_BDiv + 128) >> 8) + 128; 
		    dstpV[x] = ( ( 112 * is/m_RDiv -  94 * is/m_GDiv -  18 * is/m_BDiv + 128) >> 8) + 128 ;
		} 
	srcpU += src_pitchUV; 
	dstpU += dst_pitchUV; 
	srcpV += src_pitchUV; 
	dstpV += dst_pitchUV; 
	}
	
	for (y = 0; y < heightY; y++) 
	{
		for (int x = 0; x < row_sizeY; x++)
		{
			int is=srcpY[x]*128;
			dstpY[x] = ( (  66 * is/m_RDiv + 129 * is/m_GDiv +  25 * is/m_BDiv + 128) >> 8) +  16;
		}
	srcpY += src_pitchY; 
	dstpY += dst_pitchY; 
	} 


}

if (!processed)
	env->ThrowError("PseudoColor: Clip format not supported");   

return dst; 
}

AVSValue __cdecl Create_PColor(AVSValue args, void* user_data, IScriptEnvironment* env) {

return new PseudoColor(args[0].AsClip(),args[1].AsInt(128),args[2].AsInt(128),args[3].AsInt(128)); 
}

extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env) {

env->AddFunction("PseudoColor", "c[RDIV]i[GDIV]i[BDIV]i", Create_PColor, 0); 
return "`PseudoColor' plugin"; 
}
